﻿using DEVSsharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class Coordinator : Atomic
    {
        private List<Porcupine> porcupines;
        public List<InputPort> InputPositionPorts { get; }
        public List<InputPort> InputTempPorts { get; }
        public List<OutputPort> OutputGoToPorts { get; }
        public List<OutputPort> OutputWaitPorts { get; }

        private List<PortValue> buffer;
        private Dictionary<int, int> coupled;

        private double Tnorm;
        private double Tmin;
        private double dVid;
        private double dMin;

        public Coordinator(string name, TimeUnit tu, double dVid, double dMin, double Tmin, double Tnorm, List<Porcupine> porcupines) : base(name, tu)
        {
            this.porcupines = porcupines;
            this.Tnorm = Tnorm;
            this.Tmin = Tmin;
            this.dVid = dVid;
            this.dMin = dMin;

            InputPositionPorts = new List<InputPort>();
            InputTempPorts = new List<InputPort>();
            OutputGoToPorts = new List<OutputPort>();
            OutputWaitPorts = new List<OutputPort>();

            for(int i=0;i<porcupines.Count;i++)
            {
                InputTempPorts.Add(new InputPort("CoordinatorTempInput" + i));
                InputPositionPorts.Add(new InputPort("CoordinatorPositionInput" + i));
                OutputGoToPorts.Add(new OutputPort("CoordinatorGoToOutput" + i));
                OutputWaitPorts.Add(new OutputPort("CoordinatorWaitOutput" + i));
            }
        }

        public override bool delta_x(PortValue x)
        {
            int ind = InputPositionPorts.IndexOf(x.port as InputPort);

            if(ind!=-1)
            {
                if (porcupines[ind].T < Tmin)
                {
                    int closest = FindCloses(ind);

                    if (closest != -1)
                    {
                        if (porcupines[closest].Position.Distance(porcupines[ind].Position) < dMin)
                        {
                            buffer.Add(new PortValue(OutputWaitPorts[closest], true));
                            buffer.Add(new PortValue(OutputWaitPorts[ind], true));

                            coupled.Add(ind, closest);
                        }
                        else
                        {
                            buffer.Add(new PortValue(OutputGoToPorts[ind], porcupines[closest].Position));
                        }
                    }
                }
            }
            else
            {
                ind = InputTempPorts.IndexOf(x.port as InputPort);

                if(coupled.ContainsKey(ind) && porcupines[ind].T>Tnorm)
                {
                    buffer.Add(new PortValue(OutputWaitPorts[coupled[ind]], false));
                    buffer.Add(new PortValue(OutputWaitPorts[ind], false));

                    coupled.Remove(ind);
                }
            }

            return true;
        }

        private int FindCloses(int ind)
        {
            int closest = -1;
            Porcupine current = porcupines[ind];

            for (int i=0;i<porcupines.Count;i++)
            {
                if(i!=ind)
                {
                    if (closest == -1 || porcupines[i].Position.Distance(current.Position) < porcupines[closest].Position.Distance(current.Position))
                    {
                        closest = i;
                    }
                }
            }

            return closest;
        }

        public override void delta_y(ref PortValue y)
        {
            y = buffer[0];
            buffer.RemoveAt(0);
        }

        public override void init()
        {
            buffer = new List<PortValue>();
            coupled = new Dictionary<int, int>();
        }

        public override double tau()
        {
            return buffer.Any() ? 0 : Double.MaxValue;
        }
    }
}
