﻿using DEVSsharp;
using Stefan_Stamenkovic_91_2014.Other;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class Game : Coupled
    {
        public InputPort InputTemp { get; }
        private Random random;

        public Game(String name) : base(name)
        {
            random = new Random(DateTime.Now.Millisecond);

            List<Triangle> grid = LoadGrid();
            PrintGrid(grid);
       
            List<Porcupine> porcupines = new List<Porcupine>();

            Triangle triangle;
            Vector position = GetRandomPosition(grid, out triangle);
            porcupines.Add(new Porcupine("PorcupineA", TimeUnit.Sec, triangle, position, 0.2, 10, 2, 0.1, 20, 15, 5, 0.20, 0.15, 1.5));
            AddModel(porcupines.Last());
            
            position = GetRandomPosition(grid, out triangle);
            porcupines.Add(new Porcupine("PorcupineB", TimeUnit.Sec, triangle, position, 0.2, 10, 2, 0.1, 20, 15, 5, 0.20, 0.15, 1.5));
            AddModel(porcupines.Last());
            
            position = GetRandomPosition(grid, out triangle);
            porcupines.Add(new Porcupine("PorcupineC", TimeUnit.Sec, triangle, position, 0.2, 10, 2, 0.1, 20, 15, 5, 0.20, 0.15, 1.5));
            AddModel(porcupines.Last());
            
            position = GetRandomPosition(grid, out triangle);
            porcupines.Add(new Porcupine("PorcupineD", TimeUnit.Sec, triangle, position, 0.2, 10, 2, 0.1, 20, 15, 5, 0.20, 0.15, 1.5));
            AddModel(porcupines.Last());


            Coordinator c = new Coordinator("Coordinator", TimeUnit.Sec, 10, 1, 5, 25, porcupines);
            AddModel(c);

            Printer p = new Printer("Printer", TimeUnit.Sec, porcupines);
            AddModel(p);

            InputTemp = AddIP("InputTemp");

            for (int i=0;i<porcupines.Count;i++)
            {
                AddCP(InputTemp, porcupines[i].InputTemp);

                AddCP(porcupines[i].OutputPosition, c.InputPositionPorts[i]);
                AddCP(porcupines[i].OutputTemp, c.InputTempPorts[i]);
                AddCP(porcupines[i].OutputPosition, p.InputPrint);

                AddCP(c.OutputGoToPorts[i], porcupines[i].InputGoTo);
                AddCP(c.OutputWaitPorts[i], porcupines[i].InputWait);
            }
        }

        private Vector GetRandomPosition(List<Triangle> grid, out Triangle triangle)
        {
            int ind = random.Next(grid.Count);

            triangle = grid[ind];

            return (triangle.A + triangle.B + triangle.C) / 3;
        }

        private List<Triangle> LoadGrid()
        {
            List<Triangle> grid = new List<Triangle>();

            using (StreamReader sr = new StreamReader(File.OpenRead("./input.txt")))
            {
                while (!sr.EndOfStream)
                {
                    string[] triangleCoords = sr.ReadLine().Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

                    grid.Add(new Triangle()
                    {
                        A = Vector.Parse(triangleCoords[0].Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries)),
                        B = Vector.Parse(triangleCoords[1].Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries)),
                        C = Vector.Parse(triangleCoords[2].Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries)),
                    });
                }
            }

            InterconnectList(grid);

            return grid;
        }

        private void InterconnectList(List<Triangle> grid)
        {
            foreach (Triangle triangle in grid)
            {
                triangle.SideA = FindNeighbour(grid, triangle.B, triangle.C, triangle.A);
                triangle.SideB = FindNeighbour(grid, triangle.A, triangle.C, triangle.B);
                triangle.SideC = FindNeighbour(grid, triangle.A, triangle.B, triangle.C);
            }
        }

        private Triangle FindNeighbour(List<Triangle> grid, Vector v1, Vector v2, Vector notEqual)
        {
            return grid.FirstOrDefault(x=>
            {
                return (!x.A.Equals(notEqual) && x.B.Equals(v1) && x.C.Equals(v2))
                    || (!x.A.Equals(notEqual) && x.B.Equals(v2) && x.C.Equals(v1))
                    || (!x.B.Equals(notEqual) && x.A.Equals(v1) && x.C.Equals(v2))
                    || (!x.B.Equals(notEqual) && x.A.Equals(v2) && x.C.Equals(v1))
                    || (!x.C.Equals(notEqual) && x.A.Equals(v1) && x.B.Equals(v2))
                    || (!x.C.Equals(notEqual) && x.A.Equals(v2) && x.B.Equals(v1));
            });
        }

        private void PrintGrid(List<Triangle> grid)
        {
            int ind = 1;
            string path = "./grid.log";

            if(File.Exists(path))
            {
                File.Delete(path);
            }

            using (StreamWriter sw = new StreamWriter(File.Create(path)))
            {
                foreach (Triangle triangle in grid)
                {
                    sw.WriteLine(String.Format("{4} Triangle: {0}, Neighbours: {1} | {2} | {3}", triangle,
                        triangle.SideA != null ? triangle.SideA.ToString() : "NULL",
                        triangle.SideB != null ? triangle.SideB.ToString() : "NULL",
                        triangle.SideC != null ? triangle.SideC.ToString() : "NULL", ind));

                    ind++;
                }
            }
        }
    }
}
