﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Other
{
    class Vector
    {
        public double X { get; set; }
        public double Y { get; set; }
        public double Z { get; set; }

        public Vector() { }

        public Vector(double x, double y, double z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public double Norm()
        {
            return Math.Sqrt(X*X+Y*Y+Z*Z);
        }


        public double Distance(Vector v)
        {
            return Math.Sqrt((X - v.X) * (X - v.X) + (Y - v.Y) * (Y - v.Y)+ (Z - v.Z) * (Z - v.Z));
        }

        public Vector CrossProduct(Vector v)
        {
            return new Vector()
            {
                X = Y * v.Z - Z * v.Y,
                Y = Z * v.X - X * v.Z,
                Z = X * v.Y - Y * v.X
            };
        }

        public static Vector Parse(string[] coords)
        {
            if (coords.Length == 3)
            {
                return new Vector()
                {
                    X = Double.Parse(coords[0]),
                    Y = Double.Parse(coords[1]),
                    Z = Double.Parse(coords[2])
                };
            }

            throw new ArgumentException();
        }

        public override bool Equals(object obj)
        {
            Vector v = obj as Vector;

            if(v!=null)
            {
                return X == v.X && Y == v.Y && Z == v.Z;
            }

            return false;
        }

        public Vector Rotate(Vector around, Vector position, double fi)
        {
            return new Vector()
            {
                X = (position.X * (around.Y * around.Y + around.Z * around.Z) - around.X * (position.Y * around.Y + position.Z * around.Z - around.X * X - around.Y * Y - around.Z * Z)) * (1 - Math.Cos(fi)) + X * Math.Cos(fi) + (-position.Z * around.Y + position.Y * around.Z - around.Z * Y + around.Y * Z) * Math.Sin(fi),
                Y = (position.Y * (around.X * around.X + around.Z * around.Z) - around.Y * (position.X * around.X + position.Z * around.Z - around.X * X - around.Y * Y - around.Z * Z)) * (1 - Math.Cos(fi)) + Y * Math.Cos(fi) + (position.Z * around.X - position.X * around.Z + around.Z * X - around.Y * Z) * Math.Sin(fi),
                Z = (position.Z * (around.X * around.X + around.Y * around.Y) - around.Z * (position.X * around.X + position.Y * around.Y - around.X * X - around.Y * Y - around.Z * Z)) * (1 - Math.Cos(fi)) + Z * Math.Cos(fi) + (-position.Y * around.X + position.X * around.Y - around.Y * X + around.X * Y) * Math.Sin(fi)
            };
        }

        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }

        public override string ToString()
        {
            return String.Format("({0},{1},{2})", X, Y, Z);
        }

        public Vector Clone()
        {
            return new Vector(X,Y,Z);
        }

        public static Vector operator +(Vector v1, Vector v2)
        {
            return new Vector()
            {
                X = v1.X + v2.X,
                Y = v1.Y + v2.Y,
                Z = v1.Z + v2.Z
            };
        }

        public static Vector operator -(Vector v1, Vector v2)
        {
            return new Vector()
            {
                X = v1.X - v2.X,
                Y = v1.Y - v2.Y,
                Z = v1.Z - v2.Z
            };
        }

        public static double operator *(Vector v1, Vector v2)
        {
            return v1.X * v2.X + v1.Y * v2.Y + v1.Z * v2.Z;
        }

        public static Vector operator *(Vector v, double c)
        {
            return new Vector()
            {
                X = v.X * c,
                Y = v.Y * c,
                Z = v.Z * c
            };
        }

        public static Vector operator /(Vector v, double c)
        {
            return new Vector()
            {
                X = v.X / c,
                Y = v.Y / c,
                Z = v.Z / c
            };
        }
    }
}
