﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DEVSsharp;

namespace Rad
{
    public class Temperatura : Atomic
    {
        public double T;

        public double eps,Ts,k0,kk,ks;

        public InputPort inputT;
        public InputPort inputV;
        public double V = 0;
        public OutputPort outputT;
        public InputPort inputBlizuSam;
        public InputPort inputNisamBlizu;
        int id;
        bool deli = false;
        List<PortValue> msgBuf;

        public Temperatura(string name, TimeUnit tu, double eps, double T,int id) 
            : base(name, tu)
        {
            this.eps = eps;
            this.T = T;
            k0 = 0.1;
            kk = 1.0;
            ks = 1.0;
            Ts = 20;
            inputT = AddIP("inputT");
            inputBlizuSam = AddIP("inputBlizuSam");
            inputNisamBlizu = AddIP("inputNisamBlizu");
            inputV = AddIP("inputV");
            this.id = id;
            outputT = AddOP("outputT");
          
        }

        public override void init()
        {
           

            msgBuf = new List<PortValue>();

            msgBuf.Add(new PortValue(outputT, new TemperaturaPoruka(T,id)));
            
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port == inputV)
            {

                T += IzracunajT(TimeElapsed);
                V = (double)x.value;


                msgBuf.Add(new PortValue(outputT, new TemperaturaPoruka(T, id)));


                return true;
            }
            else if(x.port == inputT)
            {
                Ts = (double)x.value;
                return true;
            }
            else if(x.port == inputBlizuSam){
                deli = true;
                return true;
            }
            else if(x.port == inputNisamBlizu)
            {
                deli = false;
                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if (msgBuf.Count() > 0)
            {
                y = msgBuf[0];
                msgBuf.RemoveAt(0);

                return;
            }

            T += IzracunajT(TimeElapsed);

            msgBuf.Add(new PortValue(outputT, new TemperaturaPoruka(T, id)));
        }

        private double IzracunajT(double dt)
        {
            if(!deli) return dt*(k0+kk*V+ks*(Ts-T));
            return dt * (k0 + kk * V + ks/2 * (Ts - T));
        }

        public override double tau()
        {
            if (msgBuf.Count() > 0)
                return 0.0;
            if (deli)
            {
                if (k0 + kk * V + ks/2 * (Ts - T) >= 0) return eps / (k0 + kk * V + ks * (Ts - T));
                else return -eps / (k0 + kk * V + ks/2 * (Ts - T));
            }
            else
            {
                if (k0 + kk * V + ks * (Ts - T) >= 0) return eps / (k0 + kk * V + ks * (Ts - T));
                else return -eps / (k0 + kk * V + ks * (Ts - T));
            }
        }

        public override string Get_s()
        {
            return "T: " + T ;
        }
    }
}
